@@ -58,10 +58,6 @@
 #endif /* !O_NDELAY */
 #endif /* !O_NONBLOCK */
 
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
-
 #if defined(__QNX__) && !defined(__QNXNTO__)
 /* exec*() from <process.h> */
 #include <unix.h>
@@ -204,9 +200,9 @@ typedef struct
     gboolean utf8_display;
 
     /* Set if the nice message (hint) bar is visible */
-    int message_visible;
+    gboolean message_visible;
     /* Set if the nice and useful keybar is visible */
-    int keybar_visible;
+    gboolean keybar_visible;
 
 #ifdef ENABLE_BACKGROUND
     /* If true, this is a background process */
@@ -253,7 +249,7 @@ typedef struct
 #endif                          /* !ENABLE_SUBSHELL */
 
         /* This flag is set by xterm detection routine in function main() */
-        /* It is used by function view_other_cmd() */
+        /* It is used by function toggle_subshell() */
         gboolean xterm_flag;
 
         /* disable x11 support */
@@ -275,9 +271,6 @@ typedef struct
         /* If true, use + and \ keys normally and select/unselect do if M-+ / M-\.
            and M-- and keypad + / - */
         gboolean alternate_plus_minus;
-
-        /* Set if the window has changed it's size */
-        SIG_ATOMIC_VOLATILE_T winch_flag;
     } tty;
 
     struct
