@@ -1,7 +1,7 @@
 /*
    Provides a serialize/unserialize functionality for INI-like formats.
 
-   Copyright (C) 2011-2018
+   Copyright (C) 2011-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -193,7 +193,7 @@ mc_deserialize_str (const char prefix, const char *data, GError ** error)
  */
 
 char *
-mc_serialize_config (const mc_config_t * data, GError ** error)
+mc_serialize_config (mc_config_t * data, GError ** error)
 {
     gchar **groups, **group_iterator;
     GString *buffer;
