@@ -1,7 +1,7 @@
 /*
    Color setup for NCurses screen library
 
-   Copyright (C) 1994-2016
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -180,8 +180,8 @@ tty_color_try_alloc_pair_lib (tty_color_pair_t * mc_color_pair)
         ibg = mc_color_pair->ibg;
         attr = mc_color_pair->attr;
 
-        /* In non-256 color mode, change bright colors into bold */
-        if (!tty_use_256colors ())
+        /* In legacy color mode, change bright colors into bold */
+        if (!tty_use_256colors () && !tty_use_truecolors (NULL))
         {
             if (ifg >= 8 && ifg < 16)
             {
@@ -234,3 +234,13 @@ tty_use_256colors (void)
 }
 
 /* --------------------------------------------------------------------------------------------- */
+
+gboolean
+tty_use_truecolors (GError ** error)
+{
+    /* Not yet supported in ncurses */
+    g_set_error (error, MC_ERROR, -1, _("True color not supported with ncurses."));
+    return FALSE;
+}
+
+/* --------------------------------------------------------------------------------------------- */
