@@ -1,6 +1,6 @@
 
 /** \file tty-internal.h
- *  \brief Header: internal suff of the terminal controlling library
+ *  \brief Header: internal stuff of the terminal controlling library
  */
 
 #ifndef MC__TTY_INTERNAL_H
@@ -30,11 +30,17 @@ extern gboolean mouse_enabled;
 extern char *smcup;
 extern char *rmcup;
 
+/* pipe to handle SIGWINCH */
+extern int sigwinch_pipe[2];
+
 /*** declarations of public functions ************************************************************/
 
+void tty_create_winch_pipe (void);
+
 char *mc_tty_normalize_from_utf8 (const char *);
 void tty_init_xterm_support (gboolean is_xterm);
 int tty_lowlevel_getch (void);
 
 /*** inline functions ****************************************************************************/
+
 #endif /* MC_TTY_INTERNAL_H */
