@@ -2,7 +2,7 @@
    Interface to the terminal controlling library.
    Ncurses wrapper.
 
-   Copyright (C) 2005-2017
+   Copyright (C) 2005-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -102,6 +102,8 @@ tty_setup_sigwinch (void (*handler) (int))
 #endif /* SA_RESTART */
     sigaction (SIGWINCH, &act, &oact);
 #endif /* SIGWINCH */
+
+    tty_create_winch_pipe ();
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -109,9 +111,12 @@ tty_setup_sigwinch (void (*handler) (int))
 static void
 sigwinch_handler (int dummy)
 {
+    ssize_t n = 0;
+
     (void) dummy;
 
-    mc_global.tty.winch_flag = 1;
+    n = write (sigwinch_pipe[1], "", 1);
+    (void) n;
 }
 
 /* --------------------------------------------------------------------------------------------- */
