@@ -1,7 +1,7 @@
 /*
    Interface to the terminal controlling library.
 
-   Copyright (C) 2005-2018
+   Copyright (C) 2005-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -30,10 +30,18 @@
 
 #include <config.h>
 
+#include <errno.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>             /* memset() */
+
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#else
+#include <sys/time.h>
+#include <sys/types.h>
+#endif
 #include <unistd.h>             /* exit() */
 
 #ifdef HAVE_SYS_IOCTL_H
@@ -164,6 +172,49 @@ tty_got_interrupt (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
+gboolean
+tty_got_winch (void)
+{
+    fd_set fdset;
+    /* *INDENT-OFF* */
+    /* instant timeout */
+    struct timeval timeout = { .tv_sec = 0, .tv_usec = 0 };
+    /* *INDENT-ON* */
+    int ok;
+
+    FD_ZERO (&fdset);
+    FD_SET (sigwinch_pipe[0], &fdset);
+
+    while ((ok = select (sigwinch_pipe[0] + 1, &fdset, NULL, NULL, &timeout)) < 0)
+        if (errno != EINTR)
+        {
+            perror (_("Cannot check SIGWINCH pipe"));
+            exit (EXIT_FAILURE);
+        }
+
+    return (ok != 0 && FD_ISSET (sigwinch_pipe[0], &fdset));
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+void
+tty_flush_winch (void)
+{
+    ssize_t n;
+
+    /* merge all SIGWINCH events raised to this moment */
+    do
+    {
+        char x[16];
+
+        /* read multiple events at a time  */
+        n = read (sigwinch_pipe[0], &x, sizeof (x));
+    }
+    while (n > 0 || (n == -1 && errno == EINTR));
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 void
 tty_print_one_hline (gboolean single)
 {
