@@ -1,7 +1,7 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994-2018
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Authors:
@@ -74,12 +74,12 @@ check_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
         if (parm != ' ')
             return MSG_NOT_HANDLED;
         c->state = !c->state;
-        widget_redraw (w);
+        widget_draw (w);
         send_message (w->owner, w, MSG_NOTIFY, 0, NULL);
         return MSG_HANDLED;
 
     case MSG_CURSOR:
-        widget_move (w, 0, 1);
+        widget_gotoyx (w, 0, 1);
         return MSG_HANDLED;
 
     case MSG_DRAW:
@@ -88,14 +88,14 @@ check_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
 
             focused = widget_get_state (w, WST_FOCUSED);
             widget_selectcolor (w, focused, FALSE);
-            widget_move (w, 0, 0);
+            widget_gotoyx (w, 0, 0);
             tty_print_string (c->state ? "[x] " : "[ ] ");
             hotkey_draw (w, c->text, focused);
             return MSG_HANDLED;
         }
 
     case MSG_DESTROY:
-        release_hotkey (c->text);
+        hotkey_free (c->text);
         return MSG_HANDLED;
 
     default:
@@ -138,7 +138,7 @@ check_new (int y, int x, gboolean state, const char *text)
 
     c = g_new (WCheck, 1);
     w = WIDGET (c);
-    c->text = parse_hotkey (text);
+    c->text = hotkey_new (text);
     /* 4 is width of "[X] " */
     widget_init (w, y, x, 1, 4 + hotkey_width (c->text), check_callback, check_mouse_callback);
     w->options |= WOP_SELECTABLE | WOP_WANT_CURSOR | WOP_WANT_HOTKEY;
@@ -148,3 +148,29 @@ check_new (int y, int x, gboolean state, const char *text)
 }
 
 /* --------------------------------------------------------------------------------------------- */
+
+void
+check_set_text (WCheck * check, const char *text)
+{
+    Widget *w = WIDGET (check);
+    hotkey_t hk;
+
+    hk = hotkey_new (text);
+    if (hotkey_equal (check->text, hk))
+    {
+        hotkey_free (hk);
+        return;
+    }
+
+    hotkey_free (check->text);
+    check->text = hk;
+
+    if (check->text.start[0] == '\0' && check->text.hotkey == NULL && check->text.end == NULL)
+        w->cols = 3;            /* "[ ]" */
+    else
+        w->cols = 4 + hotkey_width (check->text);       /* "[ ]  text" */
+
+    widget_draw (w);
+}
+
+/* --------------------------------------------------------------------------------------------- */
