@@ -1,7 +1,7 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994-2018
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Authors:
@@ -72,7 +72,7 @@ groupbox_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
             if (g->title != NULL)
             {
                 tty_setcolor (disabled ? DISABLED_COLOR : h->color[DLG_COLOR_TITLE]);
-                widget_move (w, 0, 1);
+                widget_gotoyx (w, 0, 1);
                 tty_print_string (g->title);
             }
             return MSG_HANDLED;
@@ -124,7 +124,7 @@ groupbox_set_title (WGroupbox * g, const char *title)
         g_free (t);
     }
 
-    widget_redraw (WIDGET (g));
+    widget_draw (WIDGET (g));
 }
 
 /* --------------------------------------------------------------------------------------------- */
