@@ -1,7 +1,7 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994-2018
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Authors:
@@ -96,7 +96,7 @@ label_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
                     q[0] = '\0';
                 }
 
-                widget_move (w, y, 0);
+                widget_gotoyx (w, y, 0);
                 tty_print_string (str_fit_to_term (p, w->cols, align));
 
                 if (q == NULL)
@@ -173,7 +173,7 @@ label_set_text (WLabel * label, const char *text)
         }
     }
 
-    widget_redraw (w);
+    widget_draw (w);
 
     if (newcols < w->cols)
         w->cols = newcols;
