@@ -1,7 +1,7 @@
 /*
    Pulldown menu code
 
-   Copyright (C) 1994-2018
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -39,12 +39,15 @@
 #include "lib/tty/tty.h"
 #include "lib/skin.h"
 #include "lib/tty/key.h"        /* key macros */
+#include "lib/keybind.h"        /* global_keymap_t */
 #include "lib/strutil.h"
 #include "lib/widget.h"
 #include "lib/event.h"          /* mc_event_raise() */
 
 /*** global variables ****************************************************************************/
 
+const global_keymap_t *menu_map = NULL;
+
 /*** file scope macro definitions ****************************************************************/
 
 #define MENUENTRY(x) ((menu_entry_t *)(x))
@@ -116,9 +119,9 @@ menu_arrange (menu_t * menu, dlg_shortcut_str get_shortcut)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-menubar_paint_idx (WMenuBar * menubar, unsigned int idx, int color)
+menubar_paint_idx (const WMenuBar * menubar, unsigned int idx, int color)
 {
-    Widget *w = WIDGET (menubar);
+    const Widget *w = CONST_WIDGET (menubar);
     const menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
     const menu_entry_t *entry = MENUENTRY (g_list_nth_data (menu->entries, idx));
     const int y = 2 + idx;
@@ -132,10 +135,10 @@ menubar_paint_idx (WMenuBar * menubar, unsigned int idx, int color)
         /* menu separator */
         tty_setcolor (MENU_ENTRY_COLOR);
 
-        widget_move (w, y, x - 1);
+        widget_gotoyx (w, y, x - 1);
         tty_print_alt_char (ACS_LTEE, FALSE);
         tty_draw_hline (w->y + y, w->x + x, ACS_HLINE, menu->max_entry_len + 3);
-        widget_move (w, y, x + menu->max_entry_len + 3);
+        widget_gotoyx (w, y, x + menu->max_entry_len + 3);
         tty_print_alt_char (ACS_RTEE, FALSE);
     }
     else
@@ -144,7 +147,7 @@ menubar_paint_idx (WMenuBar * menubar, unsigned int idx, int color)
 
         /* menu text */
         tty_setcolor (color);
-        widget_move (w, y, x);
+        widget_gotoyx (w, y, x);
         tty_print_char ((unsigned char) entry->first_letter);
         tty_getyx (&yt, &xt);
         tty_draw_hline (yt, xt, ' ', menu->max_entry_len + 2);  /* clear line */
@@ -162,21 +165,21 @@ menubar_paint_idx (WMenuBar * menubar, unsigned int idx, int color)
 
         if (entry->shortcut != NULL)
         {
-            widget_move (w, y, x + menu->max_hotkey_len + 3);
+            widget_gotoyx (w, y, x + menu->max_hotkey_len + 3);
             tty_print_string (entry->shortcut);
         }
 
         /* move cursor to the start of entry text */
-        widget_move (w, y, x + 1);
+        widget_gotoyx (w, y, x + 1);
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-menubar_draw_drop (WMenuBar * menubar)
+menubar_draw_drop (const WMenuBar * menubar)
 {
-    Widget *w = WIDGET (menubar);
+    const Widget *w = CONST_WIDGET (menubar);
     const menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
     const unsigned int count = g_list_length (menu->entries);
     int column = menu->start_x - 1;
@@ -196,9 +199,9 @@ menubar_draw_drop (WMenuBar * menubar)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-menubar_set_color (WMenuBar * menubar, gboolean current, gboolean hotkey)
+menubar_set_color (const WMenuBar * menubar, gboolean current, gboolean hotkey)
 {
-    if (!widget_get_state (WIDGET (menubar), WST_FOCUSED))
+    if (!widget_get_state (CONST_WIDGET (menubar), WST_FOCUSED))
         tty_setcolor (MENU_INACTIVE_COLOR);
     else if (current)
         tty_setcolor (hotkey ? MENU_HOTSEL_COLOR : MENU_SELECTED_COLOR);
@@ -209,9 +212,9 @@ menubar_set_color (WMenuBar * menubar, gboolean current, gboolean hotkey)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-menubar_draw (WMenuBar * menubar)
+menubar_draw (const WMenuBar * menubar)
 {
-    Widget *w = WIDGET (menubar);
+    const Widget *w = CONST_WIDGET (menubar);
     GList *i;
 
     /* First draw the complete menubar */
@@ -225,7 +228,7 @@ menubar_draw (WMenuBar * menubar)
         gboolean is_selected = (menubar->selected == (gsize) g_list_position (menubar->menu, i));
 
         menubar_set_color (menubar, is_selected, FALSE);
-        widget_move (w, 0, menu->start_x);
+        widget_gotoyx (w, 0, menu->start_x);
 
         tty_print_char (' ');
         tty_print_string (menu->text.start);
@@ -246,7 +249,7 @@ menubar_draw (WMenuBar * menubar)
     if (menubar->is_dropped)
         menubar_draw_drop (menubar);
     else
-        widget_move (w, 0, MENU (g_list_nth_data (menubar->menu, menubar->selected))->start_x);
+        widget_gotoyx (w, 0, MENU (g_list_nth_data (menubar->menu, menubar->selected))->start_x);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -397,28 +400,36 @@ menubar_up (WMenuBar * menubar)
 static void
 menubar_first (WMenuBar * menubar)
 {
-    menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
-
-    if (menu->selected == 0)
-        return;
-
-    menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
-
-    menu->selected = 0;
-
-    while (TRUE)
+    if (menubar->is_dropped)
     {
-        menu_entry_t *entry;
+        menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
 
-        entry = MENUENTRY (g_list_nth_data (menu->entries, menu->selected));
+        if (menu->selected == 0)
+            return;
 
-        if ((entry == NULL) || (entry->command == CK_IgnoreKey))
-            menu->selected++;
-        else
-            break;
+        menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
+
+        menu->selected = 0;
+
+        while (TRUE)
+        {
+            menu_entry_t *entry;
+
+            entry = MENUENTRY (g_list_nth_data (menu->entries, menu->selected));
+
+            if ((entry == NULL) || (entry->command == CK_IgnoreKey))
+                menu->selected++;
+            else
+                break;
+        }
+
+        menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
+    }
+    else
+    {
+        menubar->selected = 0;
+        menubar_draw (menubar);
     }
-
-    menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -426,46 +437,92 @@ menubar_first (WMenuBar * menubar)
 static void
 menubar_last (WMenuBar * menubar)
 {
-    menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
-    const unsigned int len = g_list_length (menu->entries);
-    menu_entry_t *entry;
-
-    if (menu->selected == len - 1)
-        return;
-
-    menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
-
-    menu->selected = len;
-
-    do
+    if (menubar->is_dropped)
     {
-        menu->selected--;
-        entry = MENUENTRY (g_list_nth_data (menu->entries, menu->selected));
-    }
-    while ((entry == NULL) || (entry->command == CK_IgnoreKey));
+        menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
+        const unsigned int len = g_list_length (menu->entries);
+        menu_entry_t *entry;
 
-    menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
+        if (menu->selected == len - 1)
+            return;
+
+        menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
+
+        menu->selected = len;
+
+        do
+        {
+            menu->selected--;
+            entry = MENUENTRY (g_list_nth_data (menu->entries, menu->selected));
+        }
+        while ((entry == NULL) || (entry->command == CK_IgnoreKey));
+
+        menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
+    }
+    else
+    {
+        menubar->selected = g_list_length (menubar->menu) - 1;
+        menubar_draw (menubar);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
-menubar_handle_key (WMenuBar * menubar, int key)
+static cb_ret_t
+menubar_try_drop_menu (WMenuBar * menubar, int hotkey)
 {
-    /* Lowercase */
-    if (isascii (key))
-        key = g_ascii_tolower (key);
+    GList *i;
 
-    if (is_abort_char (key))
+    for (i = menubar->menu; i != NULL; i = g_list_next (i))
     {
-        menubar_finish (menubar);
-        return 1;
+        menu_t *menu = MENU (i->data);
+
+        if (menu->text.hotkey != NULL && hotkey == g_ascii_tolower (menu->text.hotkey[0]))
+        {
+            menubar_drop (menubar, g_list_position (menubar->menu, i));
+            return MSG_HANDLED;
+        }
     }
 
-    /* menubar help or menubar navigation */
-    switch (key)
+    return MSG_NOT_HANDLED;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static cb_ret_t
+menubar_try_exec_menu (WMenuBar * menubar, int hotkey)
+{
+    menu_t *menu;
+    GList *i;
+
+    menu = g_list_nth_data (menubar->menu, menubar->selected);
+
+    for (i = menu->entries; i != NULL; i = g_list_next (i))
     {
-    case KEY_F (1):
+        const menu_entry_t *entry = MENUENTRY (i->data);
+
+        if (entry != NULL && entry->text.hotkey != NULL
+            && hotkey == g_ascii_tolower (entry->text.hotkey[0]))
+        {
+            menu->selected = g_list_position (menu->entries, i);
+            menubar_execute (menubar);
+            return MSG_HANDLED;
+        }
+    }
+
+    return MSG_NOT_HANDLED;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static cb_ret_t
+menubar_execute_cmd (WMenuBar * menubar, unsigned long command)
+{
+    cb_ret_t ret = MSG_HANDLED;
+
+    switch (command)
+    {
+    case CK_Help:
         {
             ev_help_t event_data = { NULL, NULL };
 
@@ -477,98 +534,72 @@ menubar_handle_key (WMenuBar * menubar, int key)
 
             mc_event_raise (MCEVENT_GROUP_CORE, "help", &event_data);
             menubar_draw (menubar);
-            return 1;
         }
+        break;
 
-    case KEY_LEFT:
-    case XCTRL ('b'):
+    case CK_Left:
         menubar_left (menubar);
-        return 1;
-
-    case KEY_RIGHT:
-    case XCTRL ('f'):
+        break;
+    case CK_Right:
         menubar_right (menubar);
-        return 1;
+        break;
+    case CK_Up:
+        if (menubar->is_dropped)
+            menubar_up (menubar);
+        break;
+    case CK_Down:
+        if (menubar->is_dropped)
+            menubar_down (menubar);
+        else
+            menubar_drop (menubar, menubar->selected);
+        break;
+    case CK_Home:
+        menubar_first (menubar);
+        break;
+    case CK_End:
+        menubar_last (menubar);
+        break;
+
+    case CK_Enter:
+        if (menubar->is_dropped)
+            menubar_execute (menubar);
+        else
+            menubar_drop (menubar, menubar->selected);
+        break;
+    case CK_Quit:
+        menubar_finish (menubar);
+        break;
 
     default:
+        ret = MSG_NOT_HANDLED;
         break;
     }
 
-    if (!menubar->is_dropped)
+    return ret;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static int
+menubar_handle_key (WMenuBar * menubar, int key)
+{
+    unsigned long cmd;
+    cb_ret_t ret = MSG_NOT_HANDLED;
+
+    cmd = keybind_lookup_keymap_command (menu_map, key);
+
+    if (cmd != CK_IgnoreKey)
+        ret = menubar_execute_cmd (menubar, cmd);
+
+    if (ret != MSG_HANDLED)
     {
-        GList *i;
-
-        /* drop menu by hotkey */
-        for (i = menubar->menu; i != NULL; i = g_list_next (i))
-        {
-            menu_t *menu = MENU (i->data);
-
-            if ((menu->text.hotkey != NULL) && (key == g_ascii_tolower (menu->text.hotkey[0])))
-            {
-                menubar_drop (menubar, g_list_position (menubar->menu, i));
-                return 1;
-            }
-        }
-
-        /* drop menu by Enter or Dowwn key */
-        if (key == KEY_ENTER || key == XCTRL ('n') || key == KEY_DOWN || key == '\n')
-            menubar_drop (menubar, menubar->selected);
-
-        return 1;
+        if (menubar->is_dropped)
+            ret = menubar_try_exec_menu (menubar, key);
+        else
+            ret = menubar_try_drop_menu (menubar, key);
     }
 
-    {
-        menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
-        GList *i;
-
-        /* execute menu command by hotkey */
-        for (i = menu->entries; i != NULL; i = g_list_next (i))
-        {
-            const menu_entry_t *entry = MENUENTRY (i->data);
-
-            if ((entry != NULL) && (entry->command != CK_IgnoreKey)
-                && (entry->text.hotkey != NULL) && (key == g_ascii_tolower (entry->text.hotkey[0])))
-            {
-                menu->selected = g_list_position (menu->entries, i);
-                menubar_execute (menubar);
-                return 1;
-            }
-        }
-
-        /* menu execute by Enter or menu navigation */
-        switch (key)
-        {
-        case KEY_ENTER:
-        case '\n':
-            menubar_execute (menubar);
-            return 1;
-
-        case KEY_HOME:
-        case ALT ('<'):
-            menubar_first (menubar);
-            break;
-
-        case KEY_END:
-        case ALT ('>'):
-            menubar_last (menubar);
-            break;
-
-        case KEY_DOWN:
-        case XCTRL ('n'):
-            menubar_down (menubar);
-            break;
-
-        case KEY_UP:
-        case XCTRL ('p'):
-            menubar_up (menubar);
-            break;
-
-        default:
-            break;
-        }
-    }
-
-    return 0;
+    return ret;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -591,11 +622,10 @@ menubar_refresh (WMenuBar * menubar)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void
+static inline void
 menubar_free_menu (WMenuBar * menubar)
 {
-    if (menubar->menu != NULL)
-        g_list_free_full (menubar->menu, (GDestroyNotify) destroy_menu);
+    g_clear_list (&menubar->menu, (GDestroyNotify) destroy_menu);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -661,7 +691,7 @@ menubar_get_menu_by_x_coord (const WMenuBar * menubar, int x)
     GList *menu;
 
     for (i = 0, menu = menubar->menu;
-         menu != NULL && x > MENU (menu->data)->start_x; i++, menu = g_list_next (menu))
+         menu != NULL && x >= MENU (menu->data)->start_x; i++, menu = g_list_next (menu))
         ;
 
     /* Don't set the invalid value -1 */
@@ -842,7 +872,7 @@ menu_entry_create (const char *name, long command)
 
     entry = g_new (menu_entry_t, 1);
     entry->first_letter = ' ';
-    entry->text = parse_hotkey (name);
+    entry->text = hotkey_new (name);
     entry->command = command;
     entry->shortcut = NULL;
 
@@ -856,7 +886,7 @@ menu_entry_free (menu_entry_t * entry)
 {
     if (entry != NULL)
     {
-        release_hotkey (entry->text);
+        hotkey_free (entry->text);
         g_free (entry->shortcut);
         g_free (entry);
     }
@@ -871,7 +901,7 @@ create_menu (const char *name, GList * entries, const char *help_node)
 
     menu = g_new (menu_t, 1);
     menu->start_x = 0;
-    menu->text = parse_hotkey (name);
+    menu->text = hotkey_new (name);
     menu->entries = entries;
     menu->max_entry_len = 1;
     menu->max_hotkey_len = 0;
@@ -886,8 +916,8 @@ create_menu (const char *name, GList * entries, const char *help_node)
 void
 menu_set_name (menu_t * menu, const char *name)
 {
-    release_hotkey (menu->text);
-    menu->text = parse_hotkey (name);
+    hotkey_free (menu->text);
+    menu->text = hotkey_new (name);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -895,7 +925,7 @@ menu_set_name (menu_t * menu, const char *name)
 void
 destroy_menu (menu_t * menu)
 {
-    release_hotkey (menu->text);
+    hotkey_free (menu->text);
     g_list_free_full (menu->entries, (GDestroyNotify) menu_entry_free);
     g_free (menu->help_node);
     g_free (menu);
@@ -904,14 +934,15 @@ destroy_menu (menu_t * menu)
 /* --------------------------------------------------------------------------------------------- */
 
 WMenuBar *
-menubar_new (int y, int x, int cols, GList * menu, gboolean visible)
+menubar_new (GList * menu, gboolean visible)
 {
     WMenuBar *menubar;
     Widget *w;
 
     menubar = g_new0 (WMenuBar, 1);
     w = WIDGET (menubar);
-    widget_init (w, y, x, 1, cols, menubar_callback, menubar_mouse_callback);
+    widget_init (w, 0, 0, 1, COLS, menubar_callback, menubar_mouse_callback);
+    w->pos_flags = WPOS_KEEP_HORZ | WPOS_KEEP_TOP;
     /* initially, menubar is not selectable */
     widget_set_options (w, WOP_SELECTABLE, FALSE);
     w->options |= WOP_TOP_SELECT;
