@@ -1,7 +1,7 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994-2018
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Authors:
@@ -92,7 +92,7 @@ radio_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
             if (r->pos > 0)
             {
                 r->pos--;
-                widget_redraw (w);
+                widget_draw (w);
                 return MSG_HANDLED;
             }
             return MSG_NOT_HANDLED;
@@ -102,7 +102,7 @@ radio_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
             if (r->count - 1 > r->pos)
             {
                 r->pos++;
-                widget_redraw (w);
+                widget_draw (w);
                 return MSG_HANDLED;
             }
             return MSG_NOT_HANDLED;
@@ -112,7 +112,7 @@ radio_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
         }
 
     case MSG_CURSOR:
-        widget_move (r, r->pos, 1);
+        widget_gotoyx (r, r->pos, 1);
         return MSG_HANDLED;
 
     case MSG_DRAW:
@@ -124,7 +124,7 @@ radio_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
             for (i = 0; i < r->count; i++)
             {
                 widget_selectcolor (w, i == r->pos && focused, FALSE);
-                widget_move (w, i, 0);
+                widget_gotoyx (w, i, 0);
                 tty_draw_hline (w->y + i, w->x, ' ', w->cols);
                 tty_print_string ((r->sel == i) ? "(*) " : "( ) ");
                 hotkey_draw (w, r->texts[i], i == r->pos && focused);
@@ -135,7 +135,7 @@ radio_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
 
     case MSG_DESTROY:
         for (i = 0; i < r->count; i++)
-            release_hotkey (r->texts[i]);
+            hotkey_free (r->texts[i]);
         g_free (r->texts);
         return MSG_HANDLED;
 
@@ -188,7 +188,7 @@ radio_new (int y, int x, int count, const char **texts)
     {
         int width;
 
-        r->texts[i] = parse_hotkey (texts[i]);
+        r->texts[i] = hotkey_new (texts[i]);
         width = hotkey_width (r->texts[i]);
         wmax = MAX (width, wmax);
     }
