@@ -1,7 +1,7 @@
 /*
    Chown-advanced command -- for the Midnight Commander
 
-   Copyright (C) 1994-2018
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -246,19 +246,19 @@ print_flags (const WDialog * h)
 
     for (i = 0; i < 3; i++)
     {
-        widget_move (h, BY + 1, advanced_chown_but[0].x + 6 + i);
+        widget_gotoyx (h, BY + 1, advanced_chown_but[0].x + 6 + i);
         tty_print_char (ch_flags[i]);
     }
 
     for (i = 0; i < 3; i++)
     {
-        widget_move (h, BY + 1, advanced_chown_but[1].x + 6 + i);
+        widget_gotoyx (h, BY + 1, advanced_chown_but[1].x + 6 + i);
         tty_print_char (ch_flags[i + 3]);
     }
 
     for (i = 0; i < 3; i++)
     {
-        widget_move (h, BY + 1, advanced_chown_but[2].x + 6 + i);
+        widget_gotoyx (h, BY + 1, advanced_chown_but[2].x + 6 + i);
         tty_print_char (ch_flags[i + 6]);
     }
 
@@ -266,12 +266,12 @@ print_flags (const WDialog * h)
 
     for (i = 0; i < 15; i++)
     {
-        widget_move (h, BY + 1, advanced_chown_but[3].x + 6 + i);
+        widget_gotoyx (h, BY + 1, advanced_chown_but[3].x + 6 + i);
         tty_print_char (ch_flags[9]);
     }
     for (i = 0; i < 15; i++)
     {
-        widget_move (h, BY + 1, advanced_chown_but[4].x + 6 + i);
+        widget_gotoyx (h, BY + 1, advanced_chown_but[4].x + 6 + i);
         tty_print_char (ch_flags[10]);
     }
 }
@@ -285,19 +285,19 @@ advanced_chown_refresh (WDialog * h)
 
     tty_setcolor (COLOR_NORMAL);
 
-    widget_move (h, BY - 1, advanced_chown_but[0].x + 5);
+    widget_gotoyx (h, BY - 1, advanced_chown_but[0].x + 5);
     tty_print_string (_("owner"));
-    widget_move (h, BY - 1, advanced_chown_but[1].x + 5);
+    widget_gotoyx (h, BY - 1, advanced_chown_but[1].x + 5);
     tty_print_string (_("group"));
-    widget_move (h, BY - 1, advanced_chown_but[2].x + 5);
+    widget_gotoyx (h, BY - 1, advanced_chown_but[2].x + 5);
     tty_print_string (_("other"));
 
-    widget_move (h, BY - 1, advanced_chown_but[3].x + 5);
+    widget_gotoyx (h, BY - 1, advanced_chown_but[3].x + 5);
     tty_print_string (_("owner"));
-    widget_move (h, BY - 1, advanced_chown_but[4].x + 5);
+    widget_gotoyx (h, BY - 1, advanced_chown_but[4].x + 5);
     tty_print_string (_("group"));
 
-    widget_move (h, BY + 1, 3);
+    widget_gotoyx (h, BY + 1, 3);
     tty_print_string (_("Flag"));
     print_flags (h);
 }
@@ -501,6 +501,7 @@ chl_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *dat
         default:
             break;
         }
+        MC_FALLTHROUGH;
 
     default:
         return dlg_default_callback (w, sender, msg, parm, data);
