@@ -2,7 +2,7 @@
    Routines invoked by a function key
    They normally operate on the current panel.
 
-   Copyright (C) 1994-2017
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -90,7 +90,7 @@
 #include "command.h"            /* cmdline */
 #include "layout.h"             /* get_current_type() */
 #include "ext.h"                /* regex_command() */
-#include "boxes.h"              /* cd_dialog() */
+#include "boxes.h"              /* cd_box() */
 #include "dir.h"
 
 #include "cmd.h"                /* Our definitions */
@@ -109,7 +109,9 @@
 
 enum CompareMode
 {
-    compare_quick, compare_size_only, compare_thourough
+    compare_quick = 0,
+    compare_size_only,
+    compare_thourough
 };
 
 /*** file scope variables ************************************************************************/
@@ -133,15 +135,11 @@ do_view_cmd (gboolean normal)
     {
         vfs_path_t *fname_vpath;
 
-        if (confirm_view_dir && (current_panel->marked || current_panel->dirs_marked))
-        {
-            if (query_dialog
-                (_("Confirmation"), _("Files tagged, want to cd?"), D_NORMAL, 2,
-                 _("&Yes"), _("&No")) != 0)
-            {
-                return;
-            }
-        }
+        if (confirm_view_dir && (current_panel->marked != 0 || current_panel->dirs_marked != 0) &&
+            query_dialog (_("Confirmation"), _("Files tagged, want to cd?"), D_NORMAL, 2,
+                          _("&Yes"), _("&No")) != 0)
+            return;
+
         fname_vpath = vfs_path_from_str (selection (current_panel)->fname);
         if (!do_cd (fname_vpath, cd_exact))
             message (D_ERROR, MSG_ERROR, _("Cannot change directory"));
@@ -244,6 +242,7 @@ compare_files (const vfs_path_t * vpath1, const vfs_path_t * vpath2, off_t size)
             /* Don't have mmap() :( Even more ugly :) */
             char buf1[BUFSIZ], buf2[BUFSIZ];
             int n1, n2;
+
             rotate_dash (TRUE);
             do
             {
@@ -290,10 +289,9 @@ compare_dir (WPanel * panel, WPanel * other, enum CompareMode mode)
 
         /* Search the corresponding entry from the other panel */
         for (j = 0; j < other->dir.len; j++)
-        {
             if (strcmp (source->fname, other->dir.list[j].fname) == 0)
                 break;
-        }
+
         if (j >= other->dir.len)
             /* Not found -> mark */
             do_file_mark (panel, i, 1);
@@ -303,19 +301,17 @@ compare_dir (WPanel * panel, WPanel * other, enum CompareMode mode)
             file_entry_t *target = &other->dir.list[j];
 
             if (mode != compare_size_only)
-            {
                 /* Older version is not marked */
                 if (source->st.st_mtime < target->st.st_mtime)
                     continue;
-            }
 
             /* Newer version with different size is marked */
             if (source->st.st_size != target->st.st_size)
             {
                 do_file_mark (panel, i, 1);
                 continue;
-
             }
+
             if (mode == compare_size_only)
                 continue;
 
@@ -324,9 +320,8 @@ compare_dir (WPanel * panel, WPanel * other, enum CompareMode mode)
                 /* Thorough compare off, compare only time stamps */
                 /* Mark newer version, don't mark version with the same date */
                 if (source->st.st_mtime > target->st.st_mtime)
-                {
                     do_file_mark (panel, i, 1);
-                }
+
                 continue;
             }
 
@@ -362,6 +357,7 @@ do_link (link_type_t link_type, const char *fname)
             input_expand_dialog (_("Link"), src, MC_HISTORY_FM_LINK, "", INPUT_COMPLETE_FILENAMES);
         if (dest == NULL || *dest == '\0')
             goto cleanup;
+
         save_cwds_stat ();
 
         fname_vpath = vfs_path_from_str (fname);
@@ -393,12 +389,13 @@ do_link (link_type_t link_type, const char *fname)
             g_free (s_str);
         }
 
-        symlink_dialog (s, d, &dest, &src);
+        symlink_box (s, d, &dest, &src);
         vfs_path_free (d);
         vfs_path_free (s);
 
         if (dest == NULL || *dest == '\0' || src == NULL || *src == '\0')
             goto cleanup;
+
         save_cwds_stat ();
 
         dest_vpath = vfs_path_from_str_flags (dest, VPF_NO_CANON);
@@ -428,9 +425,6 @@ nice_cd (const char *text, const char *xtext, const char *help,
     char *machine;
     char *cd_path;
 
-    if (!SELECTED_IS_PANEL)
-        return;
-
     machine =
         input_dialog_help (text, xtext, help, history_name, INPUT_LAST_TEXT, strip_password,
                            INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_CD | INPUT_COMPLETE_HOSTNAMES |
@@ -457,11 +451,22 @@ nice_cd (const char *text, const char *xtext, const char *help,
     }
 
     {
+        panel_view_mode_t save_type;
         vfs_path_t *cd_vpath;
 
+        save_type = get_panel_type (MENU_PANEL_IDX);
+
+        if (save_type != view_listing)
+            create_panel (MENU_PANEL_IDX, view_listing);
+
         cd_vpath = vfs_path_from_str_flags (cd_path, VPF_NO_CANON);
         if (!do_panel_cd (MENU_PANEL, cd_vpath, cd_parse_command))
+        {
             message (D_ERROR, MSG_ERROR, _("Cannot chdir to \"%s\""), cd_path);
+
+            if (save_type != view_listing)
+                create_panel (MENU_PANEL_IDX, save_type);
+        }
         vfs_path_free (cd_vpath);
     }
     g_free (cd_path);
@@ -506,8 +511,8 @@ configure_panel_listing (WPanel * p, int list_format, int brief_cols, gboolean u
 static void
 switch_to_listing (int panel_index)
 {
-    if (get_display_type (panel_index) != view_listing)
-        set_display_type (panel_index, view_listing);
+    if (get_panel_type (panel_index) != view_listing)
+        create_panel (panel_index, view_listing);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -522,31 +527,30 @@ view_file_at_line (const vfs_path_t * filename_vpath, gboolean plain_view, gbool
 
     if (plain_view)
     {
-        int changed_hex_mode = 0;
-        int changed_nroff_flag = 0;
-        int changed_magic_flag = 0;
+        mcview_mode_flags_t changed_flags;
 
-        mcview_altered_hex_mode = 0;
-        mcview_altered_nroff_flag = 0;
-        mcview_altered_magic_flag = 0;
-        if (mcview_default_hex_mode)
-            changed_hex_mode = 1;
-        if (mcview_default_nroff_flag)
-            changed_nroff_flag = 1;
-        if (mcview_default_magic_flag)
-            changed_magic_flag = 1;
-        mcview_default_hex_mode = 0;
-        mcview_default_nroff_flag = 0;
-        mcview_default_magic_flag = 0;
+        mcview_clear_mode_flags (&changed_flags);
+        mcview_altered_flags.hex = FALSE;
+        mcview_altered_flags.magic = FALSE;
+        mcview_altered_flags.nroff = FALSE;
+        if (mcview_global_flags.hex)
+            changed_flags.hex = TRUE;
+        if (mcview_global_flags.magic)
+            changed_flags.magic = TRUE;
+        if (mcview_global_flags.nroff)
+            changed_flags.nroff = TRUE;
+        mcview_global_flags.hex = FALSE;
+        mcview_global_flags.magic = FALSE;
+        mcview_global_flags.nroff = FALSE;
 
         ret = mcview_viewer (NULL, filename_vpath, start_line, search_start, search_end);
 
-        if (changed_hex_mode && !mcview_altered_hex_mode)
-            mcview_default_hex_mode = 1;
-        if (changed_nroff_flag && !mcview_altered_nroff_flag)
-            mcview_default_nroff_flag = 1;
-        if (changed_magic_flag && !mcview_altered_magic_flag)
-            mcview_default_magic_flag = 1;
+        if (changed_flags.hex && !mcview_altered_flags.hex)
+            mcview_global_flags.hex = TRUE;
+        if (changed_flags.magic && !mcview_altered_flags.magic)
+            mcview_global_flags.magic = TRUE;
+        if (changed_flags.nroff && !mcview_altered_flags.nroff)
+            mcview_global_flags.nroff = TRUE;
 
         dialog_switch_process_pending ();
     }
@@ -770,6 +774,7 @@ void
 copy_cmd (void)
 {
     save_cwds_stat ();
+
     if (panel_operate (current_panel, OP_COPY, FALSE))
     {
         update_panels (UP_OPTIMIZE, UP_KEEPSEL);
@@ -784,6 +789,7 @@ void
 rename_cmd (void)
 {
     save_cwds_stat ();
+
     if (panel_operate (current_panel, OP_MOVE, FALSE))
     {
         update_panels (UP_OPTIMIZE, UP_KEEPSEL);
@@ -798,6 +804,7 @@ void
 copy_cmd_local (void)
 {
     save_cwds_stat ();
+
     if (panel_operate (current_panel, OP_COPY, TRUE))
     {
         update_panels (UP_OPTIMIZE, UP_KEEPSEL);
@@ -812,6 +819,7 @@ void
 rename_cmd_local (void)
 {
     save_cwds_stat ();
+
     if (panel_operate (current_panel, OP_MOVE, TRUE))
     {
         update_panels (UP_OPTIMIZE, UP_KEEPSEL);
@@ -855,16 +863,16 @@ mkdir_cmd (void)
         }
 
         save_cwds_stat ();
-        if (my_mkdir (absdir, 0777) == 0)
+
+        if (my_mkdir (absdir, 0777) != 0)
+            message (D_ERROR, MSG_ERROR, "%s", unix_error_string (errno));
+        else
         {
             update_panels (UP_OPTIMIZE, dir);
             repaint_screen ();
             select_item (current_panel);
         }
-        else
-        {
-            message (D_ERROR, MSG_ERROR, "%s", unix_error_string (errno));
-        }
+
         vfs_path_free (absdir);
     }
     g_free (dir);
@@ -913,13 +921,13 @@ find_cmd (void)
 void
 filter_cmd (void)
 {
-    WPanel *p;
+    if (SELECTED_IS_PANEL)
+    {
+        WPanel *p;
 
-    if (!SELECTED_IS_PANEL)
-        return;
-
-    p = MENU_PANEL;
-    set_panel_filter (p);
+        p = MENU_PANEL;
+        set_panel_filter (p);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -943,15 +951,13 @@ void
 ext_cmd (void)
 {
     vfs_path_t *extdir_vpath;
-    int dir;
+    int dir = 0;
 
-    dir = 0;
     if (geteuid () == 0)
-    {
         dir = query_dialog (_("Extension file edit"),
                             _("Which extension file you want to edit?"), D_NORMAL, 2,
                             _("&User"), _("&System Wide"));
-    }
+
     extdir_vpath = vfs_path_build_filename (mc_global.sysconfig_dir, MC_LIB_EXT, (char *) NULL);
 
     if (dir == 0)
@@ -973,6 +979,7 @@ ext_cmd (void)
         }
         do_edit (extdir_vpath);
     }
+
     vfs_path_free (extdir_vpath);
     flush_extension_file ();
 }
@@ -1043,16 +1050,13 @@ void
 edit_fhl_cmd (void)
 {
     vfs_path_t *fhlfile_vpath = NULL;
+    int dir = 0;
 
-    int dir;
-
-    dir = 0;
     if (geteuid () == 0)
-    {
         dir = query_dialog (_("Highlighting groups file edit"),
                             _("Which highlighting file you want to edit?"), D_NORMAL, 2,
                             _("&User"), _("&System Wide"));
-    }
+
     fhlfile_vpath =
         vfs_path_build_filename (mc_global.sysconfig_dir, MC_FHL_INI_FILE, (char *) NULL);
 
@@ -1075,8 +1079,8 @@ edit_fhl_cmd (void)
         }
         do_edit (fhlfile_vpath);
     }
-    vfs_path_free (fhlfile_vpath);
 
+    vfs_path_free (fhlfile_vpath);
     /* refresh highlighting rules */
     mc_fhl_free (&mc_filehighlight);
     mc_filehighlight = mc_fhl_new (TRUE);
@@ -1113,9 +1117,12 @@ hotlist_cmd (void)
         do_cd_command (cmd);
         g_free (cmd);
     }
+
     g_free (target);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 #ifdef ENABLE_VFS
 void
 vfs_list (void)
@@ -1124,7 +1131,7 @@ vfs_list (void)
     vfs_path_t *target_vpath;
 
     target = hotlist_show (LIST_VFSLIST);
-    if (!target)
+    if (target == NULL)
         return;
 
     target_vpath = vfs_path_from_str (target);
@@ -1159,10 +1166,8 @@ compare_dirs_cmd (void)
         compare_dir (other_panel, current_panel, thorough_flag);
     }
     else
-    {
         message (D_ERROR, MSG_ERROR,
                  _("Both panels should be in the listing mode\nto use this command"));
-    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1172,18 +1177,18 @@ void
 diff_view_cmd (void)
 {
     /* both panels must be in the list mode */
-    if (get_current_type () != view_listing || get_other_type () != view_listing)
-        return;
+    if (get_current_type () == view_listing && get_other_type () == view_listing)
+    {
+        if (get_current_index () == 0)
+            dview_diff_cmd (current_panel, other_panel);
+        else
+            dview_diff_cmd (other_panel, current_panel);
 
-    if (get_current_index () == 0)
-        dview_diff_cmd (current_panel, other_panel);
-    else
-        dview_diff_cmd (other_panel, current_panel);
+        if (mc_global.mc_run_mode == MC_RUN_FULL)
+            update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 
-    if (mc_global.mc_run_mode == MC_RUN_FULL)
-        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-
-    dialog_switch_process_pending ();
+        dialog_switch_process_pending ();
+    }
 }
 #endif
 
@@ -1199,27 +1204,6 @@ swap_cmd (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-void
-view_other_cmd (void)
-{
-    static int message_flag = TRUE;
-
-    if (!mc_global.tty.xterm_flag && mc_global.tty.console_flag == '\0'
-        && !mc_global.tty.use_subshell && !output_starts_shell)
-    {
-        if (message_flag)
-            message (D_ERROR, MSG_ERROR,
-                     _("Not an xterm or Linux console;\nthe panels cannot be toggled."));
-        message_flag = FALSE;
-    }
-    else
-    {
-        toggle_panels ();
-    }
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 void
 link_cmd (link_type_t link_type)
 {
@@ -1379,12 +1363,14 @@ undelete_cmd (void)
 void
 quick_cd_cmd (void)
 {
-    char *p = cd_dialog ();
+    char *p;
 
-    if (p && *p)
+    p = cd_box ();
+    if (p != NULL && *p != '\0')
     {
-        char *q = g_strconcat ("cd ", p, (char *) NULL);
+        char *q;
 
+        q = g_strconcat ("cd ", p, (char *) NULL);
         do_cd_command (q);
         g_free (q);
     }
@@ -1475,8 +1461,8 @@ dirsizes_cmd (void)
 
     for (i = 0; i < panel->dir.len; i++)
         if (S_ISDIR (panel->dir.list[i].st.st_mode)
-            && ((panel->dirs_marked && panel->dir.list[i].f.marked)
-                || !panel->dirs_marked) && !DIR_IS_DOTDOT (panel->dir.list[i].fname))
+            && ((panel->dirs_marked != 0 && panel->dir.list[i].f.marked)
+                || panel->dirs_marked == 0) && !DIR_IS_DOTDOT (panel->dir.list[i].fname))
         {
             vfs_path_t *p;
             size_t dir_count = 0;
@@ -1528,12 +1514,12 @@ save_setup_cmd (void)
 void
 info_cmd_no_menu (void)
 {
-    if (get_display_type (0) == view_info)
-        set_display_type (0, view_listing);
-    else if (get_display_type (1) == view_info)
-        set_display_type (1, view_listing);
+    if (get_panel_type (0) == view_info)
+        create_panel (0, view_listing);
+    else if (get_panel_type (1) == view_info)
+        create_panel (1, view_listing);
     else
-        set_display_type (current_panel == left_panel ? 1 : 0, view_info);
+        create_panel (current_panel == left_panel ? 1 : 0, view_info);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1541,12 +1527,12 @@ info_cmd_no_menu (void)
 void
 quick_cmd_no_menu (void)
 {
-    if (get_display_type (0) == view_quick)
-        set_display_type (0, view_listing);
-    else if (get_display_type (1) == view_quick)
-        set_display_type (1, view_listing);
+    if (get_panel_type (0) == view_quick)
+        create_panel (0, view_listing);
+    else if (get_panel_type (1) == view_quick)
+        create_panel (1, view_listing);
     else
-        set_display_type (current_panel == left_panel ? 1 : 0, view_quick);
+        create_panel (current_panel == left_panel ? 1 : 0, view_quick);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1594,7 +1580,7 @@ setup_listing_format_cmd (void)
 void
 panel_tree_cmd (void)
 {
-    set_display_type (MENU_PANEL_IDX, view_tree);
+    create_panel (MENU_PANEL_IDX, view_tree);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1602,7 +1588,7 @@ panel_tree_cmd (void)
 void
 info_cmd (void)
 {
-    set_display_type (MENU_PANEL_IDX, view_info);
+    create_panel (MENU_PANEL_IDX, view_info);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1612,7 +1598,7 @@ quick_view_cmd (void)
 {
     if (PANEL (get_panel_widget (MENU_PANEL_IDX)) == current_panel)
         change_panel ();
-    set_display_type (MENU_PANEL_IDX, view_quick);
+    create_panel (MENU_PANEL_IDX, view_quick);
 }
 
 /* --------------------------------------------------------------------------------------------- */
