@@ -6,6 +6,7 @@
 #define MC__PANEL_H
 
 #include <inttypes.h>           /* uintmax_t */
+#include <limits.h>             /* MB_LEN_MAX */
 
 #include "lib/global.h"         /* gboolean */
 #include "lib/fs.h"             /* MC_MAXPATHLEN */
@@ -21,7 +22,7 @@
 #define selection(p) (&(p->dir.list[p->selected]))
 #define DEFAULT_USER_FORMAT "half type name | size | perm"
 
-#define LIST_TYPES 4
+#define LIST_FORMATS 4
 
 #define UP_KEEPSEL ((char *) -1)
 
@@ -33,7 +34,7 @@ typedef enum
     list_brief,                 /* Name */
     list_long,                  /* Like ls -l */
     list_user                   /* User defined */
-} list_type_t;
+} list_format_t;
 
 typedef enum
 {
@@ -66,8 +67,6 @@ enum cd_enum
 
 /*** structures declarations (and typedefs of structures)*****************************************/
 
-struct format_e;
-
 typedef struct panel_field_struct
 {
     const char *id;
@@ -93,7 +92,7 @@ typedef struct
     Widget widget;
     dir_list dir;               /* Directory contents */
 
-    list_type_t list_type;      /* listing type */
+    list_format_t list_format;  /* listing type */
     int active;                 /* If panel is currently selected */
     vfs_path_t *cwd_vpath;      /* Current Working Directory */
     vfs_path_t *lwd_vpath;      /* Last Working Directory */
@@ -119,12 +118,10 @@ typedef struct
 
     gboolean user_mini_status;  /* Is user_status_format used */
     char *user_format;          /* User format */
-    char *user_status_format[LIST_TYPES];       /* User format for status line */
+    char *user_status_format[LIST_FORMATS];     /* User format for status line */
 
-    struct format_e *format;    /* Display format */
-    struct format_e *status_format;     /* Mini status format */
-
-    int format_modified;        /* If the format was changed this is set */
+    GSList *format;             /* Display format */
+    GSList *status_format;      /* Mini status format */
 
     char *panel_name;           /* The panel name */
     struct stat dir_stat;       /* Stat of current dir: used by execute () */
@@ -152,7 +149,10 @@ extern mc_fhl_t *mc_filehighlight;
 
 /*** declarations of public functions ************************************************************/
 
-WPanel *panel_new_with_dir (const char *panel_name, const vfs_path_t * vpath);
+WPanel *panel_sized_empty_new (const char *panel_name, int y, int x, int lines, int cols);
+WPanel *panel_sized_with_dir_new (const char *panel_name, int y, int x, int lines, int cols,
+                                  const vfs_path_t * vpath);
+
 void panel_clean_dir (WPanel * panel);
 
 void panel_reload (WPanel * panel);
@@ -194,11 +194,45 @@ gboolean do_cd (const vfs_path_t * new_dir_vpath, enum cd_enum cd_type);
 
 /* --------------------------------------------------------------------------------------------- */
 /*** inline functions ****************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Empty panel creation.
+ *
+ * @param panel_name name of panel for setup retieving
+ *
+ * @return new instance of WPanel
+ */
+
+static inline WPanel *
+panel_empty_new (const char *panel_name)
+{
+    /* Unknown sizes of the panel at startup */
+    return panel_sized_empty_new (panel_name, 0, 0, 1, 1);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Panel creation for specified directory.
+ *
+ * @param panel_name name of panel for setup retieving
+ * @param vpath working panel directory. If NULL then current directory is used
+ *
+ * @return new instance of WPanel
+ */
+
+static inline WPanel *
+panel_with_dir_new (const char *panel_name, const vfs_path_t * vpath)
+{
+    /* Unknown sizes of the panel at startup */
+    return panel_sized_with_dir_new (panel_name, 0, 0, 1, 1, vpath);
+}
+
+
 /* --------------------------------------------------------------------------------------------- */
 /**
  * Panel creation.
  *
- * @param panel_name the name of the panel for setup retieving
+ * @param panel_name name of panel for setup retieving
  *
  * @return new instance of WPanel
  */
@@ -206,7 +240,26 @@ gboolean do_cd (const vfs_path_t * new_dir_vpath, enum cd_enum cd_type);
 static inline WPanel *
 panel_new (const char *panel_name)
 {
-    return panel_new_with_dir (panel_name, NULL);
+    return panel_with_dir_new (panel_name, NULL);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Panel creation with specified size.
+ *
+ * @param panel_name name of panel for setup retieving
+ * @param y y coordinate of top-left corner
+ * @param x x coordinate of top-left corner
+ * @param lines vertical size
+ * @param cols horizontal size
+ *
+ * @return new instance of WPanel
+ */
+
+static inline WPanel *
+panel_sized_new (const char *panel_name, int y, int x, int lines, int cols)
+{
+    return panel_sized_with_dir_new (panel_name, y, x, lines, cols, NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
