@@ -1,7 +1,7 @@
 /*
    src - tests for execute_external_editor_or_viewer() function
 
-   Copyright (C) 2013-2018
+   Copyright (C) 2013-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -48,7 +48,7 @@ static GPtrArray *mc_config_get_string__return_value;
 
 /* @Mock */
 gchar *
-mc_config_get_string_raw (const mc_config_t * config_ignored, const gchar * group,
+mc_config_get_string_raw (mc_config_t * config_ignored, const gchar * group,
                           const gchar * param, const gchar * default_value)
 {
     char *return_value;
@@ -97,7 +97,7 @@ setup (void)
 {
     str_init_strings (NULL);
     vfs_init ();
-    init_localfs ();
+    vfs_init_localfs ();
     vfs_setup_work_dir ();
 
     mc_config_get_string__init ();
